<?
/*--------------------------------------------------------------------
 * LICENSE: FREEWARE
 * This source code is copyright Killing Moon Ltd. 2002
 *
 * You may use this code freely, but if you redistribute the source
 * code the you must abide by the following restrictions:
 * 
 * 1 - If the code is unmodified, then all the comments must remain
 *     intact and in their original form.
 *
 * 2 - If the code is modified then either:
 *   2(a) - All the original comments must be removed. You may
 *          credit the original author if you wish, but must make
 *          clear that your version is not identical to the
 *          original version.
 *   2(b) - You must add clear documentation of ALL your alterations
 *          and additions to the code in the source comments.
 *          Additionall, you must place a clear notice at the top
 *          of the file that the source code has been modified.
 *
 * Code written by Robert Tweed.
 * Email: robert@killingmoon.com
 * Website: http://www.killingmoon.com/director/
 *
 * Adapted from C reference code and the original Blowfish paper
 * which can be found at:
 * http://www.counterpane.com/blowfish.html
 *
 */


 /*
 * Abstract Blowfish Object
 * ------------------------
 * All functionality for Blowfish cipher, but cannot initialise itself
 * Used as base class for both full Blowfish and LiteFish classes
 */

class blowfish_lowlevel_class
{
	var $blowfish_ok = false;
	var $blowfish_rounds = 16;
	var $blowfish_P;
	var $blowfish_S1;
	var $blowfish_S2;
	var $blowfish_S3;
	var $blowfish_S4;

	// Simple encryption function used in each round of encryption or decryption
	function blowfish_F( $x )
	{
		$d = $x & 255;
		$x >>= 8;
		$c = $x & 255;
		$x >>= 8;
		$b = $x & 255;
		$x >>= 8;
		$a = $x & 255;

		$y = $this->blowfish_S1[ $a ] + $this->blowfish_S2[ $b ];
		$y ^= $this->blowfish_S3[ $c ];
		$y += $this->blowfish_S4[ $d ];
		
		return $y;
	}

	// Encipher a single 64-bit block
	function blowfish_encipher( &$left, &$right )
	{
		$max = $this->blowfish_rounds;
		for( $i = 0; $i < $max; ++$i )
		{
			$left ^= $this->blowfish_P[ $i ];
			$right = $this->blowfish_F( $left ) ^ $right;
			$temp = $left;
			$left = $right;
			$right = $temp;
		}

		// Undo last swap
		$temp = $left;
		$left = $right;
		$right = $temp;

		// Final Round
	  $right = $right ^ $this->blowfish_P[ $max ];
	  $left = $left ^ $this->blowfish_P[ $max+1 ];
	}

	// Decipher a single 64-bit block
	function blowfish_decipher( &$left, &$right )
	{
		for( $i = $this->blowfish_rounds+1; $i>1; --$i )
		{
			$left ^= $this->blowfish_P[ $i ];
			$right = $this->blowfish_F( $left ) ^ $right;
			$temp = $left;
			$left = $right;
			$right = $temp;
		}

		// Undo last swap
		$temp = $left;
		$left = $right;
		$right = $temp;

		// Final Round
	  $left = $left ^ $this->blowfish_P[ 0 ];
	  $right = $right ^ $this->blowfish_P[ 1 ];
	}

	// Encrypt a string and return the ciphertext as a string
	// Returned string may contain zeros
	function encrypt( $plainText )
	{
		if( ! $this->blowfish_ok ) return NULL;
		if( ! is_string( $plainText ) ) return NULL;

		$cipherText = '';

		$charCount = strlen( $plainText );
		
		for( $i = 0; $i < $charCount; $i += 8 )
		{
			$left = @ord( $plainText[ $i+3 ] ) +
			      ( @ord( $plainText[ $i+2 ] ) << 8 ) +
			      ( @ord( $plainText[ $i+1 ] ) << 16 ) +
			      ( @ord( $plainText[ $i   ] ) << 24 ) ;

			$right = @ord( $plainText[ $i+7 ] ) +
			      ( @ord( $plainText[ $i+6 ] ) << 8 ) +
			      ( @ord( $plainText[ $i+5 ] ) << 16 ) +
			      ( @ord( $plainText[ $i+4 ] ) << 24 ) ;

			$this->blowfish_encipher( $left, $right );

			$d = chr( $left & 255 );
			$left >>= 8;
			$c = chr( $left & 255 );
			$left >>= 8;
			$b = chr( $left & 255 );
			$left >>= 8;
			$a = chr( $left & 255 );
		
			$h = chr( $right & 255 );
			$right >>= 8;
			$g = chr( $right & 255 );
			$right >>= 8;
			$f = chr( $right & 255 );
			$right >>= 8;
			$e = chr( $right & 255 );

			$cipherText .= $a. $b. $c. $d. $e. $f. $g. $h;
		}

		return $cipherText;
	}

	// Decrypt a string and return the plaintext as a string
	// Returned string may contain zero-padding at the end
	// depending on whether the plaintext length was divisible by 8 characters.
	function decrypt( $cipherText )
	{
		if( ! $this->blowfish_ok ) return NULL;
		if( ! is_string( $cipherText ) ) return NULL;

		$plainText = '';

		$charCount = strlen( $cipherText );
		
		for( $i = 0; $i < $charCount; $i += 8 )
		{
			$left = @ord( $cipherText[ $i+3 ] ) +
			      ( @ord( $cipherText[ $i+2 ] ) << 8 ) +
			      ( @ord( $cipherText[ $i+1 ] ) << 16 ) +
			      ( @ord( $cipherText[ $i   ] ) << 24 ) ;

			$right = @ord( $cipherText[ $i+7 ] ) +
			      ( @ord( $cipherText[ $i+6 ] ) << 8 ) +
			      ( @ord( $cipherText[ $i+5 ] ) << 16 ) +
			      ( @ord( $cipherText[ $i+4 ] ) << 24 ) ;

			$this->blowfish_decipher( $left, $right );

			$d = chr( $left & 255 );
			$left >>= 8;
			$c = chr( $left & 255 );
			$left >>= 8;
			$b = chr( $left & 255 );
			$left >>= 8;
			$a = chr( $left & 255 );
		
			$h = chr( $right & 255 );
			$right >>= 8;
			$g = chr( $right & 255 );
			$right >>= 8;
			$f = chr( $right & 255 );
			$right >>= 8;
			$e = chr( $right & 255 );

			$plainText .= $a. $b. $c. $d. $e. $f. $g. $h;
		}

		return $plainText;
	}
}

/*
 * LiteFish Blowfish Object
 * ------------------------
 * Fully functioning Blowfish cipher but only capable of working with one key
 * The key uses precalculated subkeys which are stored in an include file.
 */

class litefish_cipher extends blowfish_lowlevel_class
{
	// Initialise the cipher from precalculated subkey arrays
	function litefish_cipher( &$defaults_object )
	{
		$this->blowfish_P = $defaults_object->blowfish_P;
		$this->blowfish_S1 = $defaults_object->blowfish_S1;
		$this->blowfish_S2 = $defaults_object->blowfish_S2;
		$this->blowfish_S3 = $defaults_object->blowfish_S3;
		$this->blowfish_S4 = $defaults_object->blowfish_S4;
		$this->blowfish_ok = true;
	}
}

/*
 * Full Blowfish Object
 * ------------------------
 * Fully functioning Blowfish cipher, which includes "setKey" method.
 * Requires the standard blowfish defaults, which are supplied in a separate include file.
 */

class blowfish_cipher extends blowfish_lowlevel_class
{
	var $defaults;

	// Initialise the cipher from precalculated subkey arrays
	function blowfish_cipher( &$defaults_object, $keyString = NULL )
	{
		$this->defaults = &$defaults_object;
		if( is_string( $keyString ) ) $this->setKey( $keyString );
	}

	// Initialise the subkeys from the specified key
	function setKey( $keyString )
	{
		$this->blowfish_ok = false;

		if( !is_string( $keyString ) ) return 'Supplied key is not a string.';

		$keyLength = strlen( $keyString );

		if( $keyLength < 4 ) return 'Supplied key is too short.';
		if( $keyLength > 56 ) return 'Supplied key is too long.';
		if( $keyLength % 4 ) return 'Supplied key is not a multiple of 32 bits in length.';

		$this->blowfish_S1 = $this->defaults->blowfish_S1;
		$this->blowfish_S2 = $this->defaults->blowfish_S2;
		$this->blowfish_S3 = $this->defaults->blowfish_S3;
		$this->blowfish_S4 = $this->defaults->blowfish_S4;

		$max = $this->blowfish_rounds + 2;
		$keyIndex = 0;
		
		for( $i = 0; $i< $max; ++$i )
		{
			$data = 0;
			for( $j = 4; $j; --$j )
			{
				$data = ( $data << 8 ) | ord( $keyString[ $keyIndex ] );
				$keyIndex = ( $keyIndex+1 ) % $keyLength;
			}
			$this->blowfish_P[ $i ] = $this->defaults->blowfish_P[ $i ] ^ $data;
		}

		$left = 0;
		$right = 0;

		for( $i = 0; $i < $max; $i += 2 )
		{
			$this->blowfish_encipher( $left, $right );
			$this->blowfish_P[ $i ] = $left;
			$this->blowfish_P[ $i+1 ] = $right;
		}

		for( $i = 0; $i < 256; $i += 2 )
		{
			$this->blowfish_encipher( $left, $right );
			$this->blowfish_S1[ $i ] = $left;
			$this->blowfish_S1[ $i+1 ] = $right;
		}

		for( $i = 0; $i < 256; $i += 2 )
		{
			$this->blowfish_encipher( $left, $right );
			$this->blowfish_S2[ $i ] = $left;
			$this->blowfish_S2[ $i+1 ] = $right;
		}

		for( $i = 0; $i < 256; $i += 2 )
		{
			$this->blowfish_encipher( $left, $right );
			$this->blowfish_S3[ $i ] = $left;
			$this->blowfish_S3[ $i+1 ] = $right;
		}

		for( $i = 0; $i < 256; $i += 2 )
		{
			$this->blowfish_encipher( $left, $right );
			$this->blowfish_S4[ $i ] = $left;
			$this->blowfish_S4[ $i+1 ] = $right;
		}

		$this->blowfish_ok = true;
	}
}
?>